/*
 * Java Genetic Algorithm Library (jenetics-2.0.2).
 * Copyright (c) 2007-2014 Franz Wilhelmstötter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author:
 *    Franz Wilhelmstötter (franz.wilhelmstoetter@gmx.at)
 */
package org.jenetics.stat;

import java.util.Random;

import org.apache.commons.math3.stat.descriptive.SummaryStatistics;
import org.testng.Assert;
import org.testng.annotations.Test;

import org.jenetics.util.Factory;
import org.jenetics.util.MappedAccumulatorTester;
import org.jenetics.util.RandomRegistry;

/**
 * @author <a href="mailto:franz.wilhelmstoetter@gmx.at">Franz Wilhelmstötter</a>
 * @version <em>$Date: 2013-12-18 $</em>
 */
public class VarianceTest extends MappedAccumulatorTester<Variance<Double>> {

	private final Factory<Variance<Double>> _factory = new Factory<Variance<Double>>() {
		@Override
		public Variance<Double> newInstance() {
			final Random random = RandomRegistry.getRandom();

			final Variance<Double> variance = new Variance<>();
			for (int i = 0; i < 1000; ++i) {
				variance.accumulate(random.nextGaussian());
			}

			return variance;
		}
	};
	@Override
	protected Factory<Variance<Double>> getFactory() {
		return _factory;
	}

	@Test
	public void variance() {
		final int length = 100_000;
		final Random random = new Random(23434);
		final SummaryStatistics stat = new SummaryStatistics();
		final Variance<Double> mean = new Variance<>();

		for (int i = 0; i < length; ++i) {
			final double value = random.nextDouble()*1_000;
			stat.addValue(value);
			mean.accumulate(value);
		}

		Assert.assertEquals(mean.getVariance(), stat.getVariance(), 0.0000001);
	}

}
